import codecs
import os

from django.core.management.base import (
    BaseCommand,
)
from django.db import (
    connection,
)
from educommon.audit_log.constants import (
    SQL_FILES_DIR,
)
from educommon.audit_log.utils import (
    get_db_connection_params,
)


class Command(BaseCommand):
    """Переустанавливает триггеры на указанные таблицы."""

    help = 'Команда для переустановки триггеров.'

    def _read_sql(self, filename):
        """Чтение SQL-кода из файла."""
        sql_file_path = os.path.join(SQL_FILES_DIR, filename)

        with codecs.open(sql_file_path, 'r', 'utf-8') as sql_file:
            sql = sql_file.read()

        self.stdout.write('reading SQL-code..\n')

        return sql

    def _prepare_sql(self, table_names_to_log):
        """Подготовка SQL-кода."""
        params = get_db_connection_params()
        params['table_names_to_log'] = table_names_to_log

        self.stdout.write('preparing SQL-code..\n')

        return self._read_sql('reinstall_triggers.sql').format(**params)

    def handle(self, *args, **options):
        """Формирование SQL-кода и его исполнение."""
        from django.apps import (
            apps,
        )

        self.stdout.write('start reinstalling triggers..\n')

        table_names_to_log = tuple({
            model._meta.db_table
            for model in apps.get_models()
            if getattr(model, 'need_to_log', False)
        })

        cursor = connection.cursor()
        sql = self._prepare_sql(table_names_to_log)
        cursor.execute(sql)

        self.stdout.write('reinstalling triggers finished.\n')
