-- Удаляет триггер audit_trigger для всех таблиц в схеме public
-- и устанавливает его только для указанных таблиц.
DO
$body$
DECLARE
    target_table RECORD;
BEGIN
-- Удаляются audit_trigger для всех таблиц в схеме public:
    FOR target_table IN
        SELECT
            event_object_table,
            trigger_schema
        FROM
            information_schema.triggers
        WHERE
            event_object_schema = 'public' AND
            trigger_name = 'audit_trigger' AND
            trigger_schema = 'public'
    LOOP
        EXECUTE
           'drop trigger if exists audit_trigger ' ||
           'on ' || target_table.trigger_schema || '.' || target_table.event_object_table;
    END LOOP;

-- Устанавливаются триггеры для таблиц, указанные в tables_names_to_log
    FOR target_table IN
        SELECT
            table_name
        FROM
            information_schema.tables
        WHERE
            table_schema = 'public' AND
            table_type = 'BASE TABLE' AND
            table_name IN {table_names_to_log}
    LOOP
        EXECUTE
            'create trigger audit_trigger after insert or update or delete ' ||
            'on public.' || target_table.table_name || ' ' ||
            'for each row execute procedure audit.on_modify()';
    END LOOP;
END
$body$
LANGUAGE plpgsql;